//---------------------------------------------------------------------------
//  COM-
// :   () 2006-2009
//      http://www.denvo.ru
// 
//          
//       (     )
//        
//  e-mail den@denvo.ru
//---------------------------------------------------------------------------
#include <windows.h>
#include <stdio.h>
#pragma hdrstop

//---------------------------------------------------------------------------
unsigned __int64 PFreq;
HANDLE ComPort;

char OldState[4] = {-1, -1, -1, -1};
unsigned __int64 OldTicks[4];
unsigned __int64 LastTick;
int LastStatus = -1;
unsigned StateMask = 0;
unsigned ShowMask = MS_CTS_ON | MS_DSR_ON | MS_RING_ON | MS_RLSD_ON;
bool GraphicOutput = true;

unsigned GetPinBit(const char *);
void ShowGraphicStatus(char * * StrPtr, unsigned CurStatus, unsigned OldStatus, unsigned Mask);
//---------------------------------------------------------------------------
#pragma argsused
int main(int argc, char* argv[])
{
  printf("\nComAnalyser v0.2b freeware (C) Den Volkov (www.denvo.ru)\n");

  // Get timer resolution
  LARGE_INTEGER pf;
  if(!QueryPerformanceFrequency(&pf))
    return printf("ERROR: High resolution timer not supported\n");
  PFreq = pf.QuadPart;

  //    
  char PortName[32] = "COM1";
  bool ShowUsage = false;
  unsigned PinBit = 0;
  for(int n = 1; n < argc; ++ n)
  {
    if(!strncmpi(argv[n], "COM", 3) && strlen(argv[n]) < 5)
    {
      strcpy(PortName, argv[n]);
    }
    else if(!strcmpi(argv[n], "-G"))
    {
      GraphicOutput = false;
    }
    else if(argv[n][0] == '+' && (PinBit = GetPinBit(argv[n]+1)) != 0)
    {
      StateMask |= PinBit;
      ShowMask |= PinBit;
    }
    else if(argv[n][0] == '-' && (PinBit = GetPinBit(argv[n]+1)) != 0)
    {
      ShowMask &= ~PinBit;
    }
    else if(!strcmp(argv[n], "/?") || !strcmp(argv[n], "-?"))
    {
      ShowUsage = true;
      break;
    }
    else
    {
      printf("ERROR: Unrecognized switch '%s'\n", argv[n]);
      ShowUsage = true;
      break;
    }
  }

  if(ShowUsage)
    return puts("Usage: ComAnalyser.exe [switches]\n"
      "  where switches are:\n"
      "  COMn -- use port COMn ( default -- COM1)\n"
      "  -G -- use report-style output (default -- pseudo-graphic oscillosope)"
      "  +pin -- waiting for pin state change (default -- any pin)\n"
      "  -pin -- do not show pin column in report (default -- show all pins)\n"
      "  pin may be CTS, DSR, RING, RLSD or ALL (all pins)\n"
      "See online manual on http://www.denvo.ru/ for more information");

  if(!StateMask)
    StateMask = MS_CTS_ON | MS_DSR_ON | MS_RING_ON | MS_RLSD_ON;

  // Open com-port
  ComPort = CreateFile(PortName, GENERIC_READ, 0, 0, OPEN_EXISTING, 0, 0);
  if(ComPort == INVALID_HANDLE_VALUE)
    return printf("ERROR: Cannot open port %s\n", PortName);
  printf("Port %s opened. To exit program press Ctrl+C\n", PortName);

  // Prepare port
  if(!SetCommMask(ComPort, EV_CTS | EV_DSR | EV_RING | EV_RLSD))
    return printf("ERROR: SetCommMask return error %d\n", GetLastError());

  // Main loop
  int LineCount = 0;
  while(true)
  {
    DWORD Event = 0, Status = 0;
    LARGE_INTEGER Ticks;

    QueryPerformanceCounter(&Ticks);
    if(!GetCommModemStatus(ComPort, &Status))
      continue;
    if(LastStatus == -1 || ((DWORD)LastStatus ^ Status) & StateMask)
    {
      static char InfoBuf[1024];
      char * InfoPtr = InfoBuf;

      double Duration = double(Ticks.QuadPart - LastTick) / PFreq * 1000.;
      if(LastStatus != -1)
        InfoPtr += sprintf(InfoPtr, " %9.2f  \n", Duration);

      if(!LineCount)
      {
        strcpy(InfoPtr, "\n"), InfoPtr += 1;
        if(ShowMask & MS_CTS_ON)
          strcpy(InfoPtr, " CTS  "), InfoPtr += 6;
        if(ShowMask & MS_DSR_ON)
          strcpy(InfoPtr, " DSR  "), InfoPtr += 6;
        if(ShowMask & MS_RING_ON)
          strcpy(InfoPtr, "RING  "), InfoPtr += 6;
        if(ShowMask & MS_RLSD_ON)
          strcpy(InfoPtr, "RLSD  "), InfoPtr += 6;
        strcpy(InfoPtr, "Duration (ms)  \n"), InfoPtr += 16;
        if(GraphicOutput)
        {
          if(ShowMask & MS_CTS_ON)
            strcpy(InfoPtr, " 0 1  "), InfoPtr += 6;
          if(ShowMask & MS_DSR_ON)
            strcpy(InfoPtr, " 0 1  "), InfoPtr += 6;
          if(ShowMask & MS_RING_ON)
            strcpy(InfoPtr, " 0 1  "), InfoPtr += 6;
          if(ShowMask & MS_RLSD_ON)
            strcpy(InfoPtr, " 0 1  "), InfoPtr += 6;
          strcpy(InfoPtr, "\n"), InfoPtr += 1;
        }
      }
      if(GraphicOutput)
      {
        if(ShowMask & MS_CTS_ON)
          ShowGraphicStatus(&InfoPtr, Status, LastStatus, MS_CTS_ON);
        if(ShowMask & MS_DSR_ON)
          ShowGraphicStatus(&InfoPtr, Status, LastStatus, MS_DSR_ON);
        if(ShowMask & MS_RING_ON)
          ShowGraphicStatus(&InfoPtr, Status, LastStatus, MS_RING_ON);
        if(ShowMask & MS_RLSD_ON)
          ShowGraphicStatus(&InfoPtr, Status, LastStatus, MS_RLSD_ON);
        strcpy(InfoPtr, "\n"), InfoPtr += 1;
        if(ShowMask & MS_CTS_ON)
          ShowGraphicStatus(&InfoPtr, Status, Status, MS_CTS_ON);
        if(ShowMask & MS_DSR_ON)
          ShowGraphicStatus(&InfoPtr, Status, Status, MS_DSR_ON);
        if(ShowMask & MS_RING_ON)
          ShowGraphicStatus(&InfoPtr, Status, Status, MS_RING_ON);
        if(ShowMask & MS_RLSD_ON)
          ShowGraphicStatus(&InfoPtr, Status, Status, MS_RLSD_ON);
      }
      else
      {
        if(ShowMask & MS_CTS_ON)
          strcpy(InfoPtr, (Status & MS_CTS_ON ? "  ON  " : " OFF  ")), InfoPtr += 6;
        if(ShowMask & MS_DSR_ON)
          strcpy(InfoPtr, (Status & MS_DSR_ON ? "  ON  " : " OFF  ")), InfoPtr += 6;
        if(ShowMask & MS_RING_ON)
          strcpy(InfoPtr, (Status & MS_RING_ON ? "  ON  " : " OFF  ")), InfoPtr += 6;
        if(ShowMask & MS_RLSD_ON)
          strcpy(InfoPtr, (Status & MS_RLSD_ON ? "  ON  " : " OFF  ")), InfoPtr += 6;
      }
      fputs(InfoBuf, stdout);

      LastTick = Ticks.QuadPart;
      ++ LineCount;
      if(LineCount > 32)
        LineCount = 0;
      LastStatus = Status;
    }

    if(!WaitCommEvent(ComPort, &Event, 0))
      Sleep(1);
  }

  return 0;
}
//---------------------------------------------------------------------------
unsigned GetPinBit(const char * str)
{
  if(!strcmpi(str, "CTS"))
    return MS_CTS_ON;
  if(!strcmpi(str, "DSR"))
    return MS_DSR_ON;
  if(!strcmpi(str, "RING"))
    return MS_RING_ON;
  if(!strcmpi(str, "RLSD"))
    return MS_RLSD_ON;
  if(!strcmpi(str, "ALL"))
    return MS_CTS_ON | MS_DSR_ON | MS_RING_ON | MS_RLSD_ON;

  return 0;
}
//---------------------------------------------------------------------------
void ShowGraphicStatus(char * * StrPtr, unsigned CurStatus, unsigned OldStatus, unsigned Mask)
{
  if(~0u == OldStatus)
    OldStatus = CurStatus;
  if(CurStatus & Mask)
  {
    if(OldStatus & Mask)
      strcpy(*StrPtr, "   \xB3  ");
    else
      strcpy(*StrPtr, " \xC0\xC4\xBF  ");
  }
  else
  {
    if(OldStatus & Mask)
      strcpy(*StrPtr, " \xDA\xC4\xD9  ");
    else
      strcpy(*StrPtr, " \xB3    ");
  }
  *StrPtr += 6;
}
//---------------------------------------------------------------------------
